@@ -44,6 +44,10 @@ G_DECLARE_FINAL_TYPE (AutoarExtractor, autoar_extractor, AUTOAR, EXTRACTOR, GObj
  **/
 #define AUTOAR_EXTRACTOR_ERROR autoar_extractor_quark()
 
+#define AUTOAR_NOT_AN_ARCHIVE_ERRNO 2013
+#define AUTOAR_EMPTY_ARCHIVE_ERRNO 2014
+#define AUTOAR_PASSPHRASE_REQUIRED_ERRNO 2015
+
 GQuark           autoar_extractor_quark                       (void);
 
 AutoarExtractor *autoar_extractor_new                         (GFile *source_file,
@@ -70,6 +74,8 @@ void             autoar_extractor_set_delete_after_extraction (AutoarExtractor *
                                                                gboolean         delete_after_extraction);
 void             autoar_extractor_set_notify_interval         (AutoarExtractor *self,
                                                                gint64           notify_interval);
+void             autoar_extractor_set_passphrase              (AutoarExtractor *self,
+                                                               const gchar     *passphrase);
 
 typedef enum {
     AUTOAR_CONFLICT_UNHANDLED = 0,
