@@ -36,6 +36,7 @@ typedef struct {
 
   gboolean cancelled_signalled;
   gboolean completed_signalled;
+  gboolean request_passphrase_signalled;
 } ExtractTestData;
 
 static void extract_test_data_free (ExtractTestData *data);
@@ -111,15 +112,17 @@ extract_test_new (const char *test_name)
   input = g_file_get_child (work_directory, "input");
   reference = g_file_get_child (work_directory, "reference");
 
-  if (g_file_query_file_type (input, G_FILE_QUERY_INFO_NONE, NULL) != G_FILE_TYPE_DIRECTORY ||
-      g_file_query_file_type (reference, G_FILE_QUERY_INFO_NONE, NULL) != G_FILE_TYPE_DIRECTORY) {
-    g_printerr ("%s: input or output directory does not exist\n", test_name);
+  if (g_file_query_file_type (input, G_FILE_QUERY_INFO_NONE, NULL) != G_FILE_TYPE_DIRECTORY) {
+    g_printerr ("%s: input directory does not exist\n", test_name);
 
     g_object_unref (input);
 
     return NULL;
   }
 
+  if (!g_file_query_exists (reference, NULL))
+    g_message ("%s: reference directory does not exist\n", test_name);
+
   output = g_file_get_child (work_directory, "output");
 
   remove_directory (output);
@@ -248,6 +251,17 @@ cancelled_handler (AutoarExtractor *extractor,
   data->cancelled_signalled = TRUE;
 }
 
+static gchar*
+request_passphrase_handler (AutoarExtractor *extractor,
+                            gpointer user_data)
+{
+  ExtractTestData *data = user_data;
+
+  data->request_passphrase_signalled = TRUE;
+
+  return NULL;
+}
+
 static ExtractTestData*
 extract_test_data_new_for_extract (AutoarExtractor *extractor)
 {
@@ -271,6 +285,8 @@ extract_test_data_new_for_extract (AutoarExtractor *extractor)
                     G_CALLBACK (error_handler), data);
   g_signal_connect (extractor, "cancelled",
                     G_CALLBACK (cancelled_handler), data);
+  g_signal_connect (extractor, "request-passphrase",
+                    G_CALLBACK (request_passphrase_handler), data);
 
   data->conflict_files = g_hash_table_new_full (g_file_hash,
                                                 (GEqualFunc) g_file_equal,
@@ -340,11 +356,11 @@ scan_directory (GFile *directory,
     file = g_queue_pop_tail (files);
     file_info = g_queue_pop_tail (file_infos);
 
-    if (scanned_callback) {
+    if (scanned_callback && file != directory) {
       scanned_callback (file, file_info, callback_data);
     }
 
-    if (g_file_info_get_file_type (file_info) == G_FILE_TYPE_DIRECTORY) {
+    if (file_info && g_file_info_get_file_type (file_info) == G_FILE_TYPE_DIRECTORY) {
       enumerator = g_file_enumerate_children (file,
                                               G_FILE_ATTRIBUTE_STANDARD_NAME","
                                               G_FILE_ATTRIBUTE_STANDARD_TYPE","
@@ -1379,6 +1395,139 @@ test_readonly_directory (void)
                                G_FILE_QUERY_INFO_NONE, NULL, NULL);
 }
 
+static void
+test_encrypted (void)
+{
+  /* arextract.zip
+   * └── arextract.txt
+   *
+   * 0 directories, 1 file
+   *
+   *
+   * ref
+   * └── arextract.txt
+   *
+   * 0 directories, 1 file
+   *
+   * passphrase is password123
+   */
+
+  g_autoptr (ExtractTest) extract_test = NULL;
+  g_autoptr (ExtractTestData) data = NULL;
+  g_autoptr (GFile) archive = NULL;
+  g_autoptr (AutoarExtractor) extractor = NULL;
+
+  extract_test = extract_test_new ("test-encrypted");
+
+  if (!extract_test) {
+    g_assert_nonnull (extract_test);
+    return;
+  }
+
+  archive = g_file_get_child (extract_test->input, "arextract.zip");
+
+  extractor = autoar_extractor_new (archive, extract_test->output);
+  autoar_extractor_set_passphrase (extractor, "password123");
+
+  data = extract_test_data_new_for_extract (extractor);
+
+  autoar_extractor_start (extractor, data->cancellable);
+
+  g_assert_cmpuint (data->number_of_files, ==, 1);
+  g_assert_no_error (data->error);
+  g_assert_true (data->completed_signalled);
+  assert_reference_and_output_match (extract_test);
+}
+
+static void
+test_encrypted_request_passphrase (void)
+{
+  /* arextract.zip
+   * └── arextract.txt
+   *
+   * 0 directories, 1 file
+   *
+   *
+   * ref
+   *
+   * 0 directories, 0 files
+   *
+   *
+   * passphrase is password123
+   */
+
+  g_autoptr (ExtractTest) extract_test = NULL;
+  g_autoptr (ExtractTestData) data = NULL;
+  g_autoptr (GFile) archive = NULL;
+  g_autoptr (AutoarExtractor) extractor = NULL;
+
+  extract_test = extract_test_new ("test-encrypted-request-passphrase");
+
+  if (!extract_test) {
+    g_assert_nonnull (extract_test);
+    return;
+  }
+
+  archive = g_file_get_child (extract_test->input, "arextract.zip");
+
+  extractor = autoar_extractor_new (archive, extract_test->output);
+
+  data = extract_test_data_new_for_extract (extractor);
+
+  autoar_extractor_start (extractor, data->cancellable);
+
+  g_assert_error (data->error, AUTOAR_EXTRACTOR_ERROR, AUTOAR_PASSPHRASE_REQUIRED_ERRNO);
+  g_assert_true (data->request_passphrase_signalled);
+  g_assert_false (data->completed_signalled);
+  assert_reference_and_output_match (extract_test);
+}
+
+static void
+test_encrypted_wrong_passphrase (void)
+{
+  /* arextract.zip
+   * └── arextract.txt
+   *
+   * 0 directories, 1 file
+   *
+   *
+   * ref
+   * └── arextract.txt
+   *
+   * 0 directories, 1 file
+   *
+   *
+   * passphrase is password123
+   */
+
+  g_autoptr (ExtractTest) extract_test = NULL;
+  g_autoptr (ExtractTestData) data = NULL;
+  g_autoptr (GFile) archive = NULL;
+  g_autoptr (AutoarExtractor) extractor = NULL;
+
+  extract_test = extract_test_new ("test-encrypted-wrong-passphrase");
+
+  if (!extract_test) {
+    g_assert_nonnull (extract_test);
+    return;
+  }
+
+  archive = g_file_get_child (extract_test->input, "arextract.zip");
+
+  extractor = autoar_extractor_new (archive, extract_test->output);
+  autoar_extractor_set_passphrase (extractor, "wrong");
+
+  data = extract_test_data_new_for_extract (extractor);
+
+  autoar_extractor_start (extractor, data->cancellable);
+
+  g_assert_cmpuint (data->number_of_files, ==, 1);
+  g_assert_error (data->error, AUTOAR_LIBARCHIVE_ERROR, -1);
+  g_assert_cmpstr (data->error->message, ==, "Incorrect passphrase");
+  g_assert_false (data->completed_signalled);
+  assert_reference_and_output_match (extract_test);
+}
+
 static void
 setup_test_suite (void)
 {
@@ -1425,6 +1574,13 @@ setup_test_suite (void)
 
   g_test_add_func ("/autoar-extract/test-readonly-directory",
                    test_readonly_directory);
+
+  g_test_add_func ("/autoar-extract/test-encrypted",
+                   test_encrypted);
+  g_test_add_func ("/autoar-extract/test-encrypted-request-passphrase",
+                   test_encrypted_request_passphrase);
+  g_test_add_func ("/autoar-extract/test-encrypted-wrong-passphrase",
+                   test_encrypted_wrong_passphrase);
 }
 
 int
